from enum import Enum, auto
import skimage.restoration
import numpy as np

class Color(Enum):
    RED = auto()
    GREEN_RIGHT = auto()
    GREEN_BOTTOM = auto()
    BLUE = auto()

    def __str__(self):
        return self.name.lower()

# Among:
# - `wavelet`
# - `bilateral`
# - `tv_chambolle`
def denoise(imageNpArray, denoiserName):
    skImageRestorationDenoise = getattr(skimage.restoration, f'denoise_{denoiserName}')

    match denoiserName:
        case 'wavelet':
            imageDenoisedNpArray = skImageRestorationDenoise(imageNpArray, rescale_sigma=True)
        case 'bilateral':
            imageDenoisedNpArray = skImageRestorationDenoise(imageNpArray, sigma_color=0.05, sigma_spatial=15)
        case 'tv_chambolle':
            imageDenoisedNpArray = skImageRestorationDenoise(imageNpArray, weight=0.2)
    return imageDenoisedNpArray

class iterativeMean:
    mean = None
    numberOfElementsInMean = 0

    def add(self, element):
        if self.mean is None:
            self.mean = element
        else:
            self.mean = ((self.mean * self.numberOfElementsInMean) + element) / (self.numberOfElementsInMean + 1)
        self.numberOfElementsInMean += 1

RAW_IMAGE_FILE_EXTENSIONS = [
    'arw',
    'nef',
]

def getRawColorChannel(raw, color):
    colorDesc = raw.color_desc.decode('ascii')
    assert colorDesc == 'RGBG'
    assert np.array_equal(raw.raw_pattern, np.array([[0, 1], [3, 2]], dtype = np.uint8))
    # RG
    # GB
    rawImageVisible = raw.raw_image_visible.copy()

    redRawImageVisible = rawImageVisible[::2, ::2]
    greenRightRawImageVisible = rawImageVisible[::2, 1::2]

    greenBottomRawImageVisible = rawImageVisible[1::2, ::2]
    blueRawImageVisible = rawImageVisible[1::2, 1::2]

    match color:
        case Color.RED:
            imageNpArray = redRawImageVisible
        case Color.GREEN_RIGHT:
            imageNpArray = greenRightRawImageVisible
        case Color.GREEN_BOTTOM:
            imageNpArray = greenBottomRawImageVisible
        case Color.BLUE:
            imageNpArray = blueRawImageVisible
    return imageNpArray

def isARawImage(imageFilePath):
    return any([imageFileName.lower().endswith(f'.{rawImageFileExtension}') for rawImageFileExtension in RAW_IMAGE_FILE_EXTENSIONS])

def escapeFilePath(filePath):
    return filePath.replace('/', '_')